//! Read-only git repository explorer library.
//!
//! This library provides a more Rustic interface for libgit2, built
//! on the `git2` bindings.  If you want more low-level access to your
//! repository, consider using that library instead.
//!
//! supergit aims to make queries into a git repo as typed and easy as
//! possible.  Start by creating a
//! [`Repository`](struct.Repository.html), and enumerating or
//! fetching [`Branch`](struct.Branch.html)es that you are interested
//! in.
//!
//! Unlike `libgit2`, this library can resolve reverse dependencies
//! between files, and their commit history.  Some of these functions
//! are very computationally intensive, and will be marked with their
//! runtime cost.

pub mod branch;

mod commit;
pub use commit::Commit;

mod diff;
pub use diff::Diff;

mod repo;
pub(crate) use repo::HashId;
pub use repo::Repository;

pub mod files;

/// Contains all core functions and types in supergit
pub mod prelude {
    pub use crate::branch::{Branch, BranchIter};
    pub use crate::files::{EntryType, Explorer, TreeEntry};
    pub use crate::{Commit, Diff, Repository};
}
