{ config, lib, pkgs, ... }:

{
  services.nginx.enable = true;
  services.nginx.virtualHosts."cloud.kookiejar.tech" = {
    serverAliases = [ "cloud.alarei.kookie.space"];
    useACMEHost = "alarei.kookie.space";
    forceSSL = true;
  };

  # Give nextcloud "core" group
  users.users.nextcloud.extraGroups = [ "core" ];

  # Enable nextcloud and php settings
  services.phpfpm.phpPackage = pkgs.php73;
  services.nextcloud = {
    enable = true;
    package = pkgs.nextcloud19;
    hostName = "cloud.kookiejar.tech";
    https = true;
    autoUpdateApps.enable = true;
    config = {
      dbtype = "pgsql";
      dbuser = "nextcloud";
      dbhost = "/run/postgresql";
      dbname = "nextcloud";
      adminpassFile = "/var/lib/nextcloud.admin.pw";
      adminuser = "spacekookie";
    };
    home = "/datacore/cloud";
  };

  # Setup postgres (currently only used by nextcloud)
  services.postgresql = {
    enable = true;
    ensureDatabases = [ "nextcloud" ];
    ensureUsers = [
      { name = "nextcloud";
        ensurePermissions."DATABASE nextcloud" = "ALL PRIVILEGES";
      }
    ];
  };

}
