{ config, pkgs, ... }:

let
  # Override git to include SVN and e-mail support. Also disable install checks
  git = (pkgs.git.override { svnSupport = true; sendEmailSupport = true; })
    .overrideAttrs ({ ...}: { doInstallCheck = false; });
in
{
  home.packages = with pkgs; [

    # Custom emacs with modules
    libkookie-emacs

    # Rust development
    rustup

    # Java/ Android development
    android-studio

    # Direnv is used to trigger lorri env
    direnv

    # Configure userspace container runner
    podman
  ];

  xdg.configFile."podman/containers/libpod.conf" = {
    text = ''
      runtime_path = ["${pkgs.runc}/bin/runc"]
      conmon_path = ["${pkgs.conmon}/bin/conmon"]
    '';
  };

  xdg.configFile."podman/containers/policy.json" = {
    text = builtins.toJSON {
      default = [ { type = "insecureAcceptAnything"; } ];
    };
  };

  xdg.configFile."podman/containers/registries.conf" = {
    text = ''
      [registries.search]
      registries = ['docker.io']
    '';
  };

  xdg.configFile."podman/containers/storage.conf" = {
    text = ''
      [storage]
      driver = "zfs"
      runroot = "/tmp/1000"
      graphroot = "/home/.local/podman
    '';
  };

  # Enable lorri service to build project environments
  services.lorri.enable = true;
}
